////////////////////////////////////////////////////////////////////////////////
// TThreadedTimer component - Heart 5.0
//
// Component which allows to have an accurate timer (based on Delphi
// Developper's Journal, issue of May 1996)
//
// by Hellix
//    products@hellix.com
//    http://www.hellix.com/
//
//  Copyright 1997-2002
////////////////////////////////////////////////////////////////////////////////
// Date of Creation: 24/01/97
//
// Modifications: (model: [<initials>, dd/mm/yy] <what has been done>)
//
// [AdB, 24/04/02] Added the Kill method
// [AdB, 15/11/03] Changed the Interval to a DWORD (was WORD)
// [AdB, 20/08/04] Added the Stop method
////////////////////////////////////////////////////////////////////////////////

#ifndef ThdTimerH
#define ThdTimerH

////////////////////////////////////////////////////////////////////////////////

#include <vcl\sysutils.hpp>
#include <vcl\controls.hpp>
#include <vcl\classes.hpp>
#include <vcl\forms.hpp>

////////////////////////////////////////////////////////////////////////////////

class TTimerThread;

class PACKAGE TThreadedTimer: public TComponent
{
   friend TTimerThread;

   private:
      bool FEnabled;
      DWORD FInterval;
      TNotifyEvent FOnTimer;
      TTimerThread *FTimerThread;
      TThreadPriority FThreadPriority;
      HANDLE killEvent;
      HANDLE stopEvent;

      void __fastcall UpdateTimer();
      void __fastcall SetEnabled(const bool Value);
      void __fastcall SetInterval(const DWORD Value);
      void __fastcall SetOnTimer(const TNotifyEvent Value);
      void __fastcall SetThreadPriority(const TThreadPriority Value);

      void __fastcall Timer();

   public:
      virtual __fastcall TThreadedTimer(TComponent *aOwner);
      virtual __fastcall ~TThreadedTimer();
      void    __fastcall Kill(void);           // AdB
      void    __fastcall Stop(void);           // AdB

   __published:
	  __property bool Enabled = {read=FEnabled, write=SetEnabled, default=false};
      __property DWORD Interval = {read=FInterval, write=SetInterval, default=1000};
      __property TNotifyEvent OnTimer = {read=FOnTimer, write=SetOnTimer, default=NULL};
      __property TThreadPriority ThreadPriority = {read=FThreadPriority, write=SetThreadPriority, default=3};
};

class TTimerThread: public TThread
{
   public:
      TThreadedTimer *OwnerTimer;

      virtual __fastcall TTimerThread(bool CreateSuspended): TThread(CreateSuspended) {};
      virtual __fastcall ~TTimerThread() {};

      virtual void __fastcall Execute();
};

////////////////////////////////////////////////////////////////////////////////

#endif

// End of file
////////////////////////////////////////////////////////////////////////////////

