//---------------------------------------------------------------------------
// This routine implements the MME calls to manage the sound card, together with
// the support for the external hardware
//---------------------------------------------------------------------------

#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "sound_io.h"

//
// Global and local static variables
//

UINT       nByteIn=BYTEIN, nByteOut=BYTEOUT;

WAVEHDR    inhdr[NINBUF], outhdr[NOUTBUF];
HWAVEIN    hin  = 0;
HWAVEOUT   hout = 0;

static int startInIdx;

// A "GUID" for working with WAVEFORMATEXTENSIBLE struct (for PCM audio stream)
static const  GUID KSDATAFORMAT_SUBTYPE_PCM = {
	  0x1,0x0000,0x0010,{0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71} };

static AudioInBuffers*  inbuf;
static AudioOutBuffers* outbuf;

	   bool sndInOpen  = false, sndOutOpen = false, restarted = false;
static bool OutClosing, InClosing, bufferAllocated = false;

//------------------------------------------------------------------------------------
bool BufAlloc(void)
{
	if(bufferAllocated) return true;

	try
	{
	  VirtualLock((inbuf  = (AudioInBuffers *) VirtualAlloc(NULL, sizeof(AudioInBuffers),
						  MEM_COMMIT, PAGE_READWRITE)), sizeof(AudioInBuffers));
	  VirtualLock((outbuf = (AudioOutBuffers *) VirtualAlloc(NULL, sizeof(AudioOutBuffers),
						  MEM_COMMIT, PAGE_READWRITE)), sizeof(AudioOutBuffers));
	}
	catch (...)
	{
		return false;
	}

	if(NULL == inbuf || NULL == outbuf) return false;
	bufferAllocated = true;

	return true;
}
//------------------------------------------------------------------------------------
void BufFree(void)
{
	if(!bufferAllocated) return;

	VirtualUnlock(inbuf,  sizeof(AudioInBuffers));
	VirtualUnlock(outbuf, sizeof(AudioOutBuffers));
	VirtualFree(inbuf,  0, MEM_RELEASE);
	VirtualFree(outbuf, 0, MEM_RELEASE);

//    delete[]  inbuf;
//    delete[]  outbuf;
}
//------------------------------------------------------------------------------------
bool SoundOpenIn(void)
{
    MMRESULT ierr;
    char err[ERRLEN];

    InClosing = false;

    if(!BufAlloc())
    {  fatal("No memory for sound buffers!");
       return false;
    }

    hin = 0;
    MainForm->inQued = 0;
    ierr = SoundStartIn();
    if (ierr != MMSYSERR_NOERROR)
    {
        if (ierr == 9) return false;
        waveInGetErrorText(ierr, (LPSTR) err, ERRLEN);
        MessageBox(NULL, err, "Audio input error",
                   MB_APPLMODAL | MB_OK | MB_ICONEXCLAMATION);
        return false;
    }

    return true;
}
//------------------------------------------------------------------------------------
void SoundCloseIn(void)
{
    SoundStopIn();
    hin = 0;
}

//------------------------------------------------------------------------------------
//
// These functions control opening and closing of the MCI device
//
//------------------------------------------------------------------------------------

MMRESULT SoundStartIn(void)
{
    int i;
    char s[128];
    MMRESULT ierr;
    WAVEFORMATEX    wf;
    WAVEFORMATEXTENSIBLE Wfmex;

// first check that we have at least *ONE* MCI device

    if (waveInGetNumDevs() <= 0) return 1;

    ZeroMemory(&wf,sizeof(wf));    /* clear the structure with 0*/
    wf.wFormatTag = WAVE_FORMAT_PCM;
    wf.nChannels = 2;
    wf.nSamplesPerSec = MainForm->srateIn;
	wf.wBitsPerSample = (WORD) (8 * nByteIn);
	MainForm->wavbits = wf.wBitsPerSample;
    wf.nBlockAlign = (WORD)((wf.nChannels * wf.wBitsPerSample) >> 3);
    wf.nAvgBytesPerSec = wf.nSamplesPerSec * wf.nBlockAlign;
    wf.cbSize = 0;

    ierr = waveInOpen((LPHWAVEIN)&hin, MainForm->devIdIn, (WAVEFORMATEX*)&wf,
                             (DWORD)SoundProcIn, 0L, CALLBACK_FUNCTION);

    if (ierr==MMSYSERR_NOERROR)  // if no errors, skip WAVEFORMATEXTENSIBLE
       goto wfok;

	ZeroMemory(&Wfmex, sizeof(WAVEFORMATEXTENSIBLE));
    Wfmex.Format = wf;  // COPY the contents of the WAVEFORMATEX .
    Wfmex.Format.cbSize     = sizeof(WAVEFORMATEXTENSIBLE);
    Wfmex.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
// Wfmex.Format.nChannels, .wBitsPerSample, .nBlockAlign,
//                     .nSamplesPerSec, .nAvgBytesPerSec already set !
    Wfmex.Samples.wValidBitsPerSample = Wfmex.Format.wBitsPerSample;
    Wfmex.SubFormat = KSDATAFORMAT_SUBTYPE_PCM;
// Now make a guess for the speaker configuration :
    Wfmex.dwChannelMask = SPEAKER_FRONT_LEFT;   // MONO ...
    if(Wfmex.Format.nChannels >= 2)             // STEREO..
       Wfmex.dwChannelMask |= SPEAKER_FRONT_RIGHT;
    if(Wfmex.Format.nChannels >= 3)             // ... and more ...
       Wfmex.dwChannelMask |= SPEAKER_FRONT_CENTER;
    if(Wfmex.Format.nChannels >= 4)
       Wfmex.dwChannelMask |= SPEAKER_LOW_FREQUENCY;
    if(Wfmex.Format.nChannels >= 5)
       Wfmex.dwChannelMask |= SPEAKER_BACK_LEFT ;
    if(Wfmex.Format.nChannels >= 6)             // typical "5.1" system
       Wfmex.dwChannelMask |= SPEAKER_BACK_RIGHT;

    ierr = waveInOpen((LPHWAVEIN)&hin, MainForm->devIdIn, (WAVEFORMATEX*)&Wfmex,
                             (DWORD)SoundProcIn, 0L, CALLBACK_FUNCTION);

    if (ierr==MMSYSERR_ALLOCATED)
    {
        fatal("Windows reports that your SoundCard is already in use by another program");
        return 9;
    }
    if (ierr==MMSYSERR_BADDEVICEID)
    {
        fatal("Bad SoundCard Device");return 9;
    }
    if (ierr==MMSYSERR_NODRIVER)
    {
        fatal("No SoundCard driver!");return 9;
    }
    if (ierr==MMSYSERR_NOMEM)
    {
        fatal("Not Enough Memory for SoundCard"); return 9;
    }
    if (ierr==WAVERR_BADFORMAT)
    {
        fatal("Your SoundCard does not support this Wave Format"); return 9;
    }
    if (ierr!=MMSYSERR_NOERROR)
    { sprintf(s, "Unspecified error in SoundStartIn, error code %d", ierr);
      fatal(s);  return ierr;
    }

// Everything ok. Our device is opened and ready to start recording
// We pass all the available buffers to MMAPI
wfok:
    for(i=0; i<NINBUF && ierr == MMSYSERR_NOERROR; i++)
    {
        ZeroMemory(&inhdr[i], sizeof(inhdr[i]));
        ZeroMemory((*inbuf)[i], BUFSIZE * 2 * nByteIn);  // two channels
        inhdr[i].lpData = (char *) (*inbuf)[i];
        inhdr[i].dwBufferLength = BUFSIZE * 2 * nByteIn;// Length in bytes
        inhdr[i].dwBytesRecorded = 0;
        inhdr[i].dwUser   = 0;
        inhdr[i].dwFlags  = 0;
        inhdr[i].dwLoops  = 0;
        inhdr[i].lpNext   = NULL;
        inhdr[i].reserved = 0;
//first prepare the waveheader
		ierr = waveInPrepareHeader(hin, &inhdr[i], sizeof(WAVEHDR));
//then add the buffer to the MMAPI queue
        ierr += waveInAddBuffer(hin, &inhdr[i], sizeof(WAVEHDR));
    }

    if(ierr!=MMSYSERR_NOERROR)
    {
        if(sndInOpen) waveInClose(hin);
        fatal("Errors during In_Buffers prepare");
    }

	return ierr;
}

//------------------------------------------------------------------------------------
void SoundStopIn(void)
{
	int i, rc;

	InClosing = true; // set this, or the thread will try
					  // to write to an invalid handle!
	waveInReset(hin);
	waveInStop(hin);

    for(i=0; i<NINBUF;  i++)
    { rc = waveInUnprepareHeader(hin,  &inhdr[i],  sizeof(WAVEHDR));
      if(rc)
        fatal("Errors during Unpreparing Input Buffers");
    }

    waveInClose(hin);
}

//------------------------------------------------------------------------------------
// This function is the In CallBack from the MM Subsystem
//------------------------------------------------------------------------------------

VOID CALLBACK SoundProcIn(HWAVEIN hwi, UINT iMsg, DWORD dwInst,
                          DWORD dwParam1, DWORD dwParam2)
{
    switch(iMsg)
    {
      case WIM_DATA:
          if(!InClosing)
          {
            MainForm->inQued++;
            SetEvent(MainForm->InBufferReady);
          }
          break;

      case WIM_OPEN:  // Message sent when input device is opened
          sndInOpen = true;
          MainForm->PIn->inIdx = 0;
          startInIdx = 0;
          break;

      case WIM_CLOSE: // Message sent when input device is closed
          sndInOpen = false;
          break;
    }
}
//------------------------------------------------------------------------------------
bool SoundOpenOut(void)
{
    MMRESULT ierr;
    char err[ERRLEN];

    OutClosing = false;

    if(!BufAlloc())
    {  fatal("No memory for sound buffers!");
       return false;
    }

    hout = 0;
    MainForm->outQued = 0;
    restarted = false;
    ierr = SoundStartOut(false);    // first try

	if (ierr != MMSYSERR_NOERROR)
    {
        SoundCloseIn();    // input streaming was already started....
        if (ierr == 9) return false;
        waveOutGetErrorText(ierr, (LPSTR) err, ERRLEN);
        MessageBox(NULL, err, "Audio output error",
                   MB_APPLMODAL | MB_OK | MB_ICONEXCLAMATION);
        return false;
    }

    return true;
}
//------------------------------------------------------------------------------------
void SoundCloseOut(void)
{
    SoundStopOut();
    hout = 0;
}

//------------------------------------------------------------------------------------
MMRESULT SoundStartOut(bool secondTry)
{
    int i;
    char s[128];
    MMRESULT ierr;
    WAVEFORMATEX    wf;
    WAVEFORMATEXTENSIBLE Wfmex;

// first check that we have at least *ONE* MCI device

    if (waveOutGetNumDevs() <= 0) return 1;

    ZeroMemory(&wf,sizeof(wf));   /* clear the structure with 0*/
    wf.wFormatTag = WAVE_FORMAT_PCM;
    wf.nChannels = 2;
    wf.nSamplesPerSec = MainForm->srateOut;

// wf.nSamplesPerSec = 8000;

    wf.wBitsPerSample = (WORD) (8 * nByteOut);
    wf.nBlockAlign = (WORD)((wf.nChannels * wf.wBitsPerSample) >> 3);
    wf.nAvgBytesPerSec = wf.nSamplesPerSec*wf.nBlockAlign;
    wf.cbSize = 0;

    ierr = waveOutOpen((LPHWAVEOUT)&hout, MainForm->devIdOut, (WAVEFORMATEX*)&wf,
                            (DWORD)SoundProcOut, 0L, CALLBACK_FUNCTION);

    if (ierr==MMSYSERR_NOERROR)  // if no errors, skip WAVEFORMATEXTENSIBLE
       goto wfok;

    ZeroMemory(&Wfmex, sizeof(WAVEFORMATEXTENSIBLE));
    Wfmex.Format = wf;  // COPY the contents of the WAVEFORMATEX .
    Wfmex.Format.cbSize     = sizeof(WAVEFORMATEXTENSIBLE);
    Wfmex.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
// Wfmex.Format.nChannels, .wBitsPerSample, .nBlockAlign,
//                     .nSamplesPerSec, .nAvgBytesPerSec already set !
    Wfmex.Samples.wValidBitsPerSample = Wfmex.Format.wBitsPerSample;
    Wfmex.SubFormat = KSDATAFORMAT_SUBTYPE_PCM;
// Now make a guess for the speaker configuration :
    Wfmex.dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT;   // MONO ...
    if(Wfmex.Format.nChannels >= 2)             // STEREO..
       Wfmex.dwChannelMask |= SPEAKER_FRONT_RIGHT;
    if(Wfmex.Format.nChannels >= 3)             // ... and more ...
       Wfmex.dwChannelMask |= SPEAKER_FRONT_CENTER;
    if(Wfmex.Format.nChannels >= 4)
       Wfmex.dwChannelMask |= SPEAKER_LOW_FREQUENCY;
    if(Wfmex.Format.nChannels >= 5)
       Wfmex.dwChannelMask |= SPEAKER_BACK_LEFT ;
    if(Wfmex.Format.nChannels >= 6)             // typical "5.1" system
       Wfmex.dwChannelMask |= SPEAKER_BACK_RIGHT;

    ierr = waveOutOpen((LPHWAVEOUT)&hout, MainForm->devIdOut, (WAVEFORMATEX*)&Wfmex,
                            (DWORD)SoundProcOut, 0L, CALLBACK_FUNCTION);

    if (ierr!=MMSYSERR_NOERROR)
       if(!secondTry) return ierr;

    if (ierr==MMSYSERR_ALLOCATED)
    {
       fatal(
       "Windows reports that your SoundCard is already in use by another program\r\
                   Please check if you did set fullduplex ON");
        return 9;
    }
    if (ierr==MMSYSERR_BADDEVICEID)
    {   fatal("Bad SoundCard Device");
        return 9;
    }
    if (ierr==MMSYSERR_NODRIVER)
    {   fatal("No SoundCard driver!");
        return 9;
    }
    if (ierr==MMSYSERR_NOMEM)
    {   fatal("Not enough memory for SoundCard");
        return 9;
    }
    if (ierr==WAVERR_BADFORMAT)
    {   fatal("Your SoundCard does not Support this Wave Format");
		return 9;
    }
    if (ierr!=MMSYSERR_NOERROR)
    { sprintf(s, "Unspecified error in SoundStartOut, error code %d", ierr);
      fatal(s);  return ierr;
    }

//the device starts playback immediately not like the recording device
//which needs another command to start recording,
//so pause it initially.
wfok:
    ierr = waveOutPause(hout);

    for(i=0; i<NOUTBUF && ierr == MMSYSERR_NOERROR; i++)
    {   ZeroMemory(&outhdr[i],sizeof(outhdr[i]));
        ZeroMemory((*outbuf)[i],OUTBUFS * 2 * nByteOut);  // two channels
        outhdr[i].lpData = (char *) (*outbuf)[i];
        outhdr[i].dwBufferLength = (DWORD) OUTBUFS * 2 * nByteOut;
        outhdr[i].dwBytesRecorded = 0L;
        outhdr[i].dwUser   = 0;     // 0 = free
        outhdr[i].dwFlags  = 0L;
        outhdr[i].dwLoops  = 0L;
        outhdr[i].lpNext   = NULL;
        outhdr[i].reserved = 0L;
        ierr = waveOutPrepareHeader(hout, &outhdr[i], sizeof(WAVEHDR));
//        outhdr[i].dwFlags  |= WHDR_DONE;
    }

    if(ierr!=MMSYSERR_NOERROR)
    {
      if(sndOutOpen) waveOutClose(hout);
      fatal("Errors during Out_Buffers prepare");
      return 1;
    }

    sndOutOpen = true;
    return ierr;
}
//------------------------------------------------------------------------------------
void SoundStopOut(void)
{
    int i, rc;

    OutClosing = true;     // set this, or the thread will try
                           // to write to an invalid handle!
    sndOutOpen = false;
    waveOutPause(hout);
    waveOutReset(hout);

    for(i=0; i<NOUTBUF;  i++)
    { rc = waveOutUnprepareHeader(hout,  &outhdr[i],  sizeof(WAVEHDR));
      if(rc)
        fatal("Errors during Unpreparing Output Buffers");
    }

    waveOutClose(hout);
}
//-----------------------------------------------------------------------------
// This function is the Out CallBack from the MM Subsystem
//-----------------------------------------------------------------------------
VOID CALLBACK SoundProcOut(HWAVEOUT hwo, UINT iMsg, DWORD dwInst,
                                         DWORD dwParam1, DWORD dwParam2)
{
    switch(iMsg)
    {
      case WOM_DONE:
        if (!OutClosing)
		{
		  ((LPWAVEHDR) dwParam1)->dwUser = 0;  // 0 means free
		   MainForm->outQued--; // decrease the count of buffers queued for output;
        }
        break;

      case WOM_OPEN:  // Message sent when output device is opened
		break;

	  case WOM_CLOSE: // Message sent when output device is closed
//        sndOutOpen = false;
        break;

    }
}
//------------------------------------------------------------------------------------
int  __fastcall GetExtIOdata(void)
{
	int k;

	if(MainForm->extHWtype == USBdata24)
	{
	  k = MainForm->USBfifo24->getnum();
	  if(k > 0)
	  {
		MainForm->USBfifo24->getp(MainForm->extIOdata);
		return MainForm->numextIQpairs / BUFSIZE;    // return # of inQued buffers
	  }
	}
	else
	  if(MainForm->extHWtype == USBdata32  || MainForm->extHWtype == USBfloat32)
	  {
		k = MainForm->USBfifo32->getnum();
		if(k > 0)
		{
		  MainForm->USBfifo32->getp(MainForm->extIOdata);
		  return MainForm->numextIQpairs / BUFSIZE;  // return # of inQued buffers
		}
	  }
	  else
	  {
		k = MainForm->USBfifo->getnum();
		if(k > 0)
		{
		  MainForm->USBfifo->getp((short *)MainForm->extIOdata);
		  return MainForm->numextIQpairs / BUFSIZE;  // return # of inQued buffers
		}
	  }

	  return 0;
}
//------------------------------------------------------------------------------------
void __fastcall ReQueIn(int idx)
{
	int rc;

	if(!MainForm->extio)
	{
	  rc = waveInAddBuffer(hin, &inhdr[idx], sizeof(WAVEHDR));
	  if(rc != MMSYSERR_NOERROR)
		fatal("Errors when requeing input buffers...");
	}
}
//------------------------------------------------------------------------------------
void __fastcall ReQueOut(int idx)
{
    int rc, limit;

    rc = waveOutWrite(hout, &outhdr[idx], sizeof(WAVEHDR));
    if(rc != MMSYSERR_NOERROR)
      fatal("Errors when requeing output buffers...");

    MainForm->outQued++;
	limit = OUTBUFFERMARK;
    if(!restarted)
      if(MainForm->outQued > limit)
      { rc = waveOutRestart(hout);
        restarted = true;
        if(rc != MMSYSERR_NOERROR)
          fatal("Errors when starting the output stream ...");
      }
}
//------------------------------------------------------------------------------------
void __fastcall ResetOut(void)
{
    while(MainForm->outQued > 0) Sleep(64);
	waveOutPause(hout);
	restarted = false;
	MainForm->justStarted = true;
}
//------------------------------------------------------------------------------------
bool __fastcall getBufPtrIn(int& idx, void* inL, void* inR)
{
	int j, k;

	if(MainForm->extio)
	{
	  switch(MainForm->extHWtype)
	  {
		case SDR14     :
		case USBdata16 :
			 if(MainForm->PIn->inIdx >= MainForm->numextIQpairs * 2)
				 MainForm->PIn->inIdx = 0;

			 for(k = 0, j = 0; k < BUFSIZE; k++)
			 {
			   ((int *)inL)[k] = ((short *)MainForm->extIOdata)[MainForm->PIn->inIdx + j++] << 8; // shift to 24 bit
			   ((int *)inR)[k] = ((short *)MainForm->extIOdata)[MainForm->PIn->inIdx + j++] << 8;
			 }

			 MainForm->PIn->inIdx += BUFSIZE * 2;
			 return true;

		case USBdata24 :
			 if(MainForm->PIn->inIdx >= MainForm->numextIQpairs * 2 * 3)
				 MainForm->PIn->inIdx = 0;

			 for(k = 0, j = 0; k < BUFSIZE; k++)
			 {
				int m = MainForm->PIn->inIdx + j;
				((int *)inL)[k] = (((char *)MainForm->extIOdata)[m+2] >= 0) ? 0 : -1;
				memcpy((char *)&((int *)inL)[k], &((char *)MainForm->extIOdata)[m], 3);
				j += 3;

				m += 3;
				((int *)inR)[k] = (((char *)MainForm->extIOdata)[m+2] >= 0) ? 0 : -1;
				memcpy((char *)&((int *)inR)[k], &((char *)MainForm->extIOdata)[m], 3);
				j += 3;
			 }

			 MainForm->PIn->inIdx += BUFSIZE * 2 * 3;
			 return true;

		case USBdata32 :
			 if(MainForm->PIn->inIdx >= MainForm->numextIQpairs * 2)
				 MainForm->PIn->inIdx = 0;

			 for(k = 0, j = 0; k < BUFSIZE; k++)
			 {
			   ((int *)inL)[k] = ((int *)MainForm->extIOdata)[MainForm->PIn->inIdx + j++];
			   ((int *)inR)[k] = ((int *)MainForm->extIOdata)[MainForm->PIn->inIdx + j++];
			 }

			 MainForm->PIn->inIdx += BUFSIZE * 2;
			 return true;

		case USBfloat32 :
			 if(MainForm->PIn->inIdx >= MainForm->numextIQpairs * 2)
				 MainForm->PIn->inIdx = 0;

			 for(k = 0, j = 0; k < BUFSIZE; k++)
			 {
			   ((float *)inL)[k] = ((float *)MainForm->extIOdata)[MainForm->PIn->inIdx + j++];
			   ((float *)inR)[k] = ((float *)MainForm->extIOdata)[MainForm->PIn->inIdx + j++];
			 }

			 MainForm->PIn->inIdx += BUFSIZE * 2;
			 return true;

		case SDRX   :
		case SCdata :
			 fatal("System Error....!!! ExtIO asserted, while it shouldn't");
			 Application->Terminate();
			 break;
	  }
	}

	do   // inIdx starts from startInIdx always
	{
	  if(sndInOpen && (inhdr[MainForm->PIn->inIdx].dwFlags & WHDR_DONE)) // EQ 1 means filled
	  {
		for(k = 0, j = 0; k < BUFSIZE; k++)
		{
		   ((int *)inL)[k] = *((short *)inhdr[MainForm->PIn->inIdx].lpData + j++) << 8; // shift to 24 bit
		   ((int *)inR)[k] = *((short *)inhdr[MainForm->PIn->inIdx].lpData + j++) << 8;
        }
        idx = MainForm->PIn->inIdx;
        if(++MainForm->PIn->inIdx >= NINBUF) MainForm->PIn->inIdx = 0;
        startInIdx = MainForm->PIn->inIdx;
        return true;
      }

      if(++MainForm->PIn->inIdx >= NINBUF) MainForm->PIn->inIdx = 0;
    } while(MainForm->PIn->inIdx != startInIdx);

    return false;   // if no buffers filled, return false
}
//------------------------------------------------------------------------------------
short * __fastcall getBufPtrOut(int& idx)
{
    for(idx = 0; idx < NOUTBUF; idx++)
      if(sndOutOpen && (outhdr[idx].dwUser == 0)) // 0 means free
      {
        outhdr[idx].dwUser = 1;  // 1 means filled
        return (short *)outhdr[idx].lpData;
      }

    return NULL;   // if no buffers free, return NULL
}
//------------------------------------------------------------------------------------
void extIOCallback(int cnt, int status, float IQoffs, void *IQdata)
{

    if(cnt < 0)
    {
      static int previous = 0;

      MainForm->extHWstatus = status;
      if(MainForm->extHWtype == SDR14)
      {
        switch(status)
        {
		   case 0 /*NCON*/ :
					MainForm->SDR14unplugged = true;
					if(status != previous)
						MainForm->SelExtHW->Caption = "SDR-14/IQ not connected or powered off";
					break;

		   case 1 /*IDLE*/:
					MainForm->SDR14unplugged = false;
//                    if(MainForm->running)
//                        MainForm->SDR14err = true;
					if(status != previous)
					{
					  String s(MainForm->extHWname);
					  MainForm->SelExtHW->Caption = s.SubString(0,6) + " Ready";

					}
					break;

		   case 2 /*RUNNING*/ :
		   case 4 /*OVERLOAD*/:
					MainForm->SDR14unplugged = false;
					break;

		   case 3 /*ERROR*/   :
					MainForm->SDR14err = true;
					break;

		}
		previous = status;
	  }

	  switch(status)
	  {
		case 0 :
		case 1 :
		case 2 :
		case 3 :
		case 4 :
				break;

		case 100 : // sampling speed has changed in the external HW

				   MainForm->srchanged = true;
				   break;

		case 101 : // LO frequency has changed in the external HW

				   MainForm->extLOchanged = true;
				   break;

		case 102 : MainForm->LOlocked = true;  break;

		case 103 : MainForm->LOlocked = false; break;

		case 104 : // LO freq. has changed, Winrad must keep the Tune freq. unchanged
				   // (must immediately call GetHWLO() )

				   break;

		case 105 : // a change of the Tune freq. is being requested.
				   // Winrad must call GetTune() to know which value is wanted

				   MainForm->mustGetTune = true; break;

		case 106 : // a change of demod. mode is being requested.
				   // Winrad must call GetMode() to know the new mode

				   MainForm->mustGetMode = true; break;

		case 107 : // The DLL wants Winrad to Start

                   MainForm->StartRequested = true;
				   break;

		case 108 : // The DLL wants Winrad to Stop

				   MainForm->StopRequested = true;
				   break;

		case 109 : // a change in the band limits is being requested
				   // Winrad must call GetFilters()

				   MainForm->mustGetFilters = true; break;
	  }

	  return;

	}  // if(cnt < 0)

//-------------------- not a status change, but a sending of I/O data ------------------------

	if(!MainForm->running) return;

	if(MainForm->extHWtype == SDR14 || MainForm->extHWtype == USBdata16 ||
									   MainForm->extHWtype == USBdata24 ||
									   MainForm->extHWtype == USBdata32 ||
									   MainForm->extHWtype == USBfloat32)
	{
	  if(MainForm->extHWtype == SDR14 && status != 2 /*RUNNING*/)
		return;

	  if(IQdata == NULL) return;

	  if(MainForm->extHWtype == USBdata24)
		MainForm->USBfifo24->put(IQdata);
	  else
		if(MainForm->extHWtype == USBdata32 || MainForm->extHWtype == USBfloat32)
		  MainForm->USBfifo32->put(IQdata);
		else
		  MainForm->USBfifo->put((short *)IQdata);

	  MainForm->IQoffset = IQoffs * 256;

	  SetEvent(MainForm->InBufferReady);
    }
}
//------------------------------------------------------------------------------------
void __fastcall GetCurrentCard()
{
   String KeyName("Software\\digiRadio"), cardnameIn, cardnameOut, asioName,
          timeString;
   Byte KeyValue[KEY_LEN];
   TRegistry& SoundReg = *(new TRegistry);

   SoundReg.Access   = KEY_ALL_ACCESS;
   SoundReg.RootKey  = HKEY_CURRENT_USER;
   SoundReg.OpenKey(KeyName, true);

   cardnameIn  = SoundReg.ReadString("CurrentCardIn");
   cardnameOut = SoundReg.ReadString("CurrentCardOut");
   asioName    = SoundReg.ReadString("ASIO_Name");

   if((cardnameIn.Length() == 0 || cardnameOut.Length() == 0) &&
       asioName.Length() == 0)
   {
      MainForm->SelCardButtClick(NULL);

      SoundReg.WriteInteger("CurrentIDIn",    MainForm->devIdIn);
      SoundReg.WriteInteger("CurrentIDOut",   MainForm->devIdOut);
      SoundReg.WriteString ("CurrentCardIn",  MainForm->devNameIn);
      SoundReg.WriteString ("CurrentCardOut", MainForm->devNameOut);
      SoundReg.WriteString ("ASIO_Name",      MainForm->asioName);
      SoundReg.WriteInteger("ASIO_ID",        MainForm->asioNum);
      SoundReg.WriteString ("Version",        VERSION);
      MainForm->version = VERSION;
   }
   else
   {
	  MainForm->devIdIn  = SoundReg.ReadInteger("CurrentIDIn");
	  MainForm->devIdOut = SoundReg.ReadInteger("CurrentIDOut");
	  MainForm->asioNum  = SoundReg.ReadInteger("ASIO_ID");
	  strcpy(MainForm->devNameIn,  cardnameIn.c_str());
	  strcpy(MainForm->devNameOut, cardnameOut.c_str());
	  strcpy(MainForm->asioName,   asioName.c_str());
   }

   SoundReg.CloseKey();
   delete &SoundReg;
}
//------------------------------------------------------------------------------------
void __fastcall PutCurrentCard(void)
{
   String KeyName("Software\\digiRadio"), cardnameIn, cardnameOut;
   Byte KeyValue[KEY_LEN];
   TRegistry& SoundReg = *(new TRegistry);

   SoundReg.Access   = KEY_ALL_ACCESS;
   SoundReg.RootKey  = HKEY_CURRENT_USER;
   SoundReg.OpenKey(KeyName, true);

   SoundReg.WriteInteger("CurrentIDIn",    MainForm->devIdIn);
   SoundReg.WriteInteger("CurrentIDOut",   MainForm->devIdOut);
   SoundReg.WriteString ("CurrentCardIn",  MainForm->devNameIn);
   SoundReg.WriteString ("CurrentCardOut", MainForm->devNameOut);
   SoundReg.WriteString ("ASIO_Name",      MainForm->asioName);
   SoundReg.WriteInteger("ASIO_ID",        MainForm->asioNum);
   SoundReg.WriteString ("Version",        VERSION);

   SoundReg.CloseKey();
   delete &SoundReg;
}
//------------------------------------------------------------------------------------

