//---------------------------------------------------------------------------
// My implementation of some ASIO routines
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "myAsio.h"

//---------------------------------------------------------------------------
bool __fastcall TMainForm::RunASIO()
{
// set the sample rate
	if(ASIOSetSampleRate(sampleRate) != ASE_OK)
	{
	   sprintf(s, "The current ASIO driver does not support %d Hz as sampling rate",
																	   srateIn);
	   warn(s);
	   ASIODisposeBuffers();
	   ASIOExit();
	   asioDrivers->removeCurrentDriver();
	   return false;
	}

	asioDriverInfo.sampleRate = sampleRate;

// start the streaming
	if(ASIOStart() != ASE_OK)
	{
	   warn("Error when starting ASIO...");
	   ASIODisposeBuffers();
	   ASIOExit();
	   asioDrivers->removeCurrentDriver();
	   return false;
	}

	return true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopASIO()
{
	ASIOStop();
	DisposeASIO();
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::StartASIO()
{

	if(!InitializeASIO(false))  // false means not dummy, i.e. a real start
	   return false;;

	if(!RunASIO()) return false;

	return true;
}
//---------------------------------------------------------------------------
ASIOError __fastcall TMainForm::create_asio_buffers (DriverInfo *asioDriverInfo)
{
// create buffers for all inputs and outputs of the card with the
// preferredSize from ASIOGetBufferSize() as buffer size
	long i;
	ASIOError result;

	// fill the bufferInfos from the start without a gap
	ASIOBufferInfo *info = asioDriverInfo->bufferInfos;

	// prepare inputs (Though this is not necessaily required, no opened inputs will work, too
	if (asioDriverInfo->inputChannels > kMaxInputChannels)
		asioDriverInfo->inputBuffers = kMaxInputChannels;
	else
		asioDriverInfo->inputBuffers = asioDriverInfo->inputChannels;
	for(i = 0; i < asioDriverInfo->inputBuffers; i++, info++)
	{
		info->isInput = ASIOTrue;
		info->channelNum = i;
		info->buffers[0] = info->buffers[1] = 0;
	}

	// prepare outputs
	if (asioDriverInfo->outputChannels > kMaxOutputChannels)
		asioDriverInfo->outputBuffers = kMaxOutputChannels;
	else
		asioDriverInfo->outputBuffers = asioDriverInfo->outputChannels;
	for(i = 0; i < asioDriverInfo->outputBuffers; i++, info++)
	{
		info->isInput = ASIOFalse;
		info->channelNum = i;
		info->buffers[0] = info->buffers[1] = 0;
	}

	// create and activate buffers
	result = ASIOCreateBuffers(asioDriverInfo->bufferInfos,
		asioDriverInfo->inputBuffers + asioDriverInfo->outputBuffers,
		asioDriverInfo->preferredSize, &asioCallbacks_);
	if (result == ASE_OK)
	{
		// now get all the buffer details, sample word length, name, word clock group and activation
		for (i = 0; i < asioDriverInfo->inputBuffers + asioDriverInfo->outputBuffers; i++)
		{
			asioDriverInfo->channelInfos[i].channel = asioDriverInfo->bufferInfos[i].channelNum;
			asioDriverInfo->channelInfos[i].isInput = asioDriverInfo->bufferInfos[i].isInput;
			result = ASIOGetChannelInfo(&asioDriverInfo->channelInfos[i]);
			if (result != ASE_OK)
				break;
		}

		if (result == ASE_OK)
		{
					 char s[128];

			// get the input and output latencies
			// Latencies often are only valid after ASIOCreateBuffers()
			// (input latency is the age of the first sample in the currently returned audio block)
			// (output latency is the time the first sample in the currently returned audio block requires to get to the output)
			result = ASIOGetLatencies(&asioDriverInfo->inputLatency, &asioDriverInfo->outputLatency);
		}
	}
	return result;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::CheckASIO()
{
	if(!InitializeASIO(true))   // true => dummy request, not a real start
		return false;

	DisposeASIO();
	return true;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::InitializeASIO(bool dummy)
{
	if(wavin || extio)          // should never occurr......
	  sampleRate = (float)srateOut;
	else
	  sampleRate = (float)srateIn;

	if (loadAsioDriver (asioName))
	{
	   memset( &asioDriverInfo.driverInfo, 0, sizeof(ASIODriverInfo) );
	   asioDriverInfo.driverInfo.asioVersion = 2;
	   asioDriverInfo.driverInfo.sysRef = windowhandle;
	}
	else
	{
	   warn("Unable to load the currently selected ASIO driver...");
	   return false;
	}

// initialize the driver
	if (!myASIOInit (&asioDriverInfo.driverInfo) == ASE_OK)
	{
	   warn("Unable to initialize the ASIO driver");
	   asioDrivers->removeCurrentDriver();
	   return false;
	}

	for(int k=0; k<NUMRATES; k++)
	   asioRateSupported[k] = (ASIOCanSampleRate(asioRates[k]) == ASE_OK);

	S8000In->Enabled   = asioRateSupported[0];
	S11025In->Enabled  = asioRateSupported[1];
	S16000In->Enabled  = asioRateSupported[2];
	S22050In->Enabled  = asioRateSupported[3];
	S24000In->Enabled  = asioRateSupported[4];
	S32000In->Enabled  = asioRateSupported[5];
	S44100In->Enabled  = asioRateSupported[6];
	S48000In->Enabled  = asioRateSupported[7];
	S88200In->Enabled  = asioRateSupported[8];
	S96000In->Enabled  = asioRateSupported[9];
	S176400In->Enabled = asioRateSupported[10];
	S192000In->Enabled = asioRateSupported[11];

	for(int k=0; k<NUMRATES; k++)
	{
	   if(!asioRateSupported[k] && sampleRate == asioRates[k]) // no good...
	   {
		  int j;

		  sampleRate = 44100;      // just in case nothing is found, prevents divide by zero later...
		  for(j=0; j<NUMRATES; j++)
			if(asioRateSupported[j])
			{
			   if(asioRates[j] ==  8000.)  S8000InClick(this);
			   if(asioRates[j] == 11025.)  S11025InClick(this);
			   if(asioRates[j] == 16000.)  S16000InClick(this);
			   if(asioRates[j] == 22050.)  S22050InClick(this);
			   if(asioRates[j] == 24000.)  S24000InClick(this);
			   if(asioRates[j] == 32000.)  S32000InClick(this);
			   if(asioRates[j] == 44100.)  S44100InClick(this);
               if(asioRates[j] == 48000.)  S48000InClick(this);
               if(asioRates[j] == 88200.)  S88200InClick(this);
			   if(asioRates[j] == 96000.)  S96000InClick(this);
			   if(asioRates[j] == 176400.) S176400InClick(this);
               if(asioRates[j] == 192000.) S192000InClick(this);
               break;
            }

          if(j < NUMRATES) sampleRate = asioRates[j];   // found in the table
          srateIn = sampleRate;
          SetParam(this);
          break;
       }
    }

// set up the asioCallback structure and create the ASIO data buffer
	asioCallbacks_.bufferSwitch = bufferSwitch;
	asioCallbacks_.sampleRateDidChange = sampleRateChanged;
	asioCallbacks_.asioMessage = asioMessages;
    asioCallbacks_.bufferSwitchTimeInfo = bufferSwitchTimeInfo;

// check whether the driver requires the ASIOOutputReady() optimization
// (can be used by the driver to reduce output latency by one block)
    asioDriverInfo.postOutput = (ASIOOutputReady() == ASE_OK );

// get the number of available channels
    if(ASIOGetChannels(&asioDriverInfo.inputChannels, &asioDriverInfo.outputChannels) != ASE_OK)
    {
       warn("Error when querying number of channels...");
       return false;
    }
    inputChannelCount  = asioDriverInfo.inputChannels;
    outputChannelCount = asioDriverInfo.outputChannels;

// query input channel info
    for(int numCh=0; numCh<inputChannelCount; numCh++)
    {
      asioDriverInfo.channelInfos[numCh].channel = numCh;
      asioDriverInfo.channelInfos[numCh].isInput = true;
      if(ASIOGetChannelInfo(&asioDriverInfo.channelInfos[numCh]) != ASE_OK)
      {
         warn("Error when querying input channel info...");
         return false;
      }
    }

// query output channel info
    for(int numCh=0; numCh<outputChannelCount; numCh++)
    {
      int idx = numCh + inputChannelCount;
      asioDriverInfo.channelInfos[idx].channel = numCh;
      asioDriverInfo.channelInfos[idx].isInput = false;
      if(ASIOGetChannelInfo(&asioDriverInfo.channelInfos[idx]) != ASE_OK)
      {
         warn("Error when querying output channel info...");
         return false;
      }
    }

// query buffer sizes
    if(ASIOGetBufferSize(&asioDriverInfo.minSize, &asioDriverInfo.maxSize,
                         &asioDriverInfo.preferredSize,
                         &asioDriverInfo.granularity) != ASE_OK )
    {
       warn("Error when querying buffer size...");
       return false;
    }

    if(!dummy) PIn->wreadlen = asioDriverInfo.preferredSize;

// create ASIO buffers
    if(create_asio_buffers(&asioDriverInfo) != ASE_OK)
    {
       warn("Error when creating buffers...");
       ASIOExit();
       asioDrivers->removeCurrentDriver();
       return false;
    }

// try to locate the first two input buffers and the first two output buffers
    LChI = RChI = LChO = RChO = -1;
    for(int numCh=0; numCh<inputChannelCount + outputChannelCount; numCh++)
    {
       if(asioDriverInfo.bufferInfos[numCh].isInput)
         if(LChI < 0) LChI = numCh;
         else if(RChI < 0) RChI = numCh; else;
       else
         if(LChO < 0) LChO = numCh;
         else if(RChO < 0) RChO = numCh;
    }

    switch(asioDriverInfo.channelInfos[0].type) //let's suppose the sample type is the same on all channels...
    {
      case ASIOSTInt16LSB :   asioSampleLength = 2; break;
      case ASIOSTInt24LSB :   asioSampleLength = 3; break;
      case ASIOSTInt32LSB :
      case ASIOSTFloat32LSB : asioSampleLength = 4; break;
      case ASIOSTFloat64LSB : asioSampleLength = 8; break;

      default :
           asioSampleLength = 4; break;
    }

    ZeroMemory(asioDriverInfo.bufferInfos[LChO].buffers[0], asioDriverInfo.preferredSize * asioSampleLength);
    ZeroMemory(asioDriverInfo.bufferInfos[LChO].buffers[1], asioDriverInfo.preferredSize * asioSampleLength);
    ZeroMemory(asioDriverInfo.bufferInfos[RChO].buffers[0], asioDriverInfo.preferredSize * asioSampleLength);
    ZeroMemory(asioDriverInfo.bufferInfos[RChO].buffers[1], asioDriverInfo.preferredSize * asioSampleLength);
    ZeroMemory(asioDriverInfo.bufferInfos[LChI].buffers[0], asioDriverInfo.preferredSize * asioSampleLength);
    ZeroMemory(asioDriverInfo.bufferInfos[LChI].buffers[1], asioDriverInfo.preferredSize * asioSampleLength);
    ZeroMemory(asioDriverInfo.bufferInfos[RChI].buffers[0], asioDriverInfo.preferredSize * asioSampleLength);
    ZeroMemory(asioDriverInfo.bufferInfos[RChI].buffers[1], asioDriverInfo.preferredSize * asioSampleLength);

    return true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DisposeASIO()
{
    ASIODisposeBuffers();
    ASIOExit();

    asioDrivers->removeCurrentDriver();
}
//---------------------------------------------------------------------------

#pragma package(smart_init)
